@@ -5,6 +5,7 @@ tv.coverperc <- function (db, obs, RelScale, tv_home, tvscale, pa = FALSE, ...)
       tv_home <- tv.home(...)
   if(missing(tvscale)) 
       tvscale <- read.dbf(paste(tv_home, "Popup", "tvscale.dbf", sep = "/"))
+  tvscale <- tvscale[!is.na(tvscale$SCALE_NR),]
   rownames(tvscale) <- tvscale[, 1]
   if (missing(RelScale)) {
       ow <- options('warn')
@@ -22,25 +23,24 @@ tv.coverperc <- function (db, obs, RelScale, tv_home, tvscale, pa = FALSE, ...)
     }
   obs <- split(obs, g, drop = FALSE)
   for (i in names(obs)) {
-      if (i == "00") 
-          obs[[i]] <- data.frame(obs[[i]], COVER_PERC = as.numeric(as.character(obs[[i]][, "COVER_CODE"])))
-      else {
-    p <- which(is.na(tvscale[i,]))[1]
-    if(is.na(p)) p <- ncol(tvscale)
-    scala <- tvscale[i,]
-#    code <- apply(scala[,seq(4,p-1,2)],1,function(x) levels(factor(x)))
-    code <- t(scala[seq(4,(p-1),2)])
-#match(
-    perc <- scala[seq(5,p,2)][1,]
-    if (any(is.na(perc))) stop("TVScale.dbf is incorrect, contains NA values!")
-          d.f <- data.frame(code=code[,1], perc = as.numeric(perc))
-          cat('\n Cover code used: ', as.character(tvscale[i, 2]), '\n')
-          write.table(t(d.f), col.names = FALSE, sep = "\t", quote = FALSE)
-          obs[[i]]["COVER_PERC"] <- d.f$perc[match(obs[[i]][,"COVER_CODE"], d.f$code)]
-      }
+    if (i == "00") 
+      obs[[i]] <- data.frame(obs[[i]], COVER_PERC = as.numeric(as.character(obs[[i]][, "COVER_CODE"])))
+    else {
+      p <- which(is.na(tvscale[i,]))[1]
+      if(is.na(p)) p <- ncol(tvscale)
+      scala <- tvscale[i,]
+      if(is.na(scala[1])) stop('Can not find cover scale', i, 'in Turbowin/Popup/tvscale.dbf')
+      code <- t(scala[seq(4,(p-1),2)])
+      perc <- scala[seq(5,p,2)][1,]
+      d.f <- data.frame(code=code[,1], perc = as.numeric(perc))
+      cat('\n Cover code used: ', as.character(tvscale[i, 2]), '\n')
+      write.table(t(d.f), col.names = FALSE, sep = "\t", quote = FALSE)
+      obs[[i]]["COVER_PERC"] <- d.f$perc[match(obs[[i]][,"COVER_CODE"], d.f$code)]
+  }
   }
   obs <- unsplit(obs, g)
   }
+  if(any(is.na(obs$COVER_PERC))) stop("Invalid cover codes, please check tvabund.dbf and tvscale.dbf!")
   obs
 }
 
